/*
 * http://github.com/dusty-nv/jetson-inference
 */

#include "cudaUtility.h"
#include <iostream>



// Just a dummy cuda function
__global__ void first(float* input, float* output, int batchSize)
{
    printf("%f %f\n", output[0], output[1]);
    output[0] = 0.0;
    output[1] = 1.0;
    output[2] = 2.0;
    output[3] = 3.0;
    output[4] = 4.0;
    output[5] = 5.0;
    output[6] = 6.0;
    output[7] = 7.0;
}

void go_kernel(float* input, float* output, int batchSize, cudaStream_t stream)
{
    printf("go_kernel\n");
    dim3 dimBlock(1, 1);
    dim3 dimGrid(1, 1);

    first <<< dimGrid, dimBlock, 0, stream >>> (input, output, batchSize);
}


/*******
NCHW -> NHWC
********/
__global__ void kernel_flatten(float* input, float* output, int N, int C, int H, int W)
{
    // uint n = blockIdx.x;
    // uint c = blockIdx.y;
    // uint h = threadIdx.x;
    // uint w = threadIdx.y;

    uint n = blockIdx.x;
    uint h = blockIdx.y;
    uint w = blockIdx.z;
    uint c = threadIdx.x;

    // uint z = threadIdx.z;
    // c += 512/4*z;
    // printf("%d %d %d %d: %d <-- %d\n", n, h, w, c, n*H*W*C + h*W*C + w*C + c, n*C*H*W + c*H*W + h*W + w);

    output[n*H*W*C + h*W*C + w*C + c] = input[n*C*H*W + c*H*W + h*W + w];
}


void run_flatten(float* input, float* output, int N, int C, int H, int W, cudaStream_t stream)
{
    // dim3 dimGrid(N, C);
    // dim3 dimBlock(H, W);
    dim3 dimGrid(N, H, W);
    dim3 dimBlock(C);
    kernel_flatten <<< dimGrid, dimBlock, 0, stream >>> (input, output, N, C, H, W);
}



void* safeCudaMalloc(size_t memSize);
__global__ void kernel_channel_scale(float* output, float* scaler, int N, int H, int W, int C)
{
    // uint c = blockIdx.x;
    uint n = blockIdx.x;
    uint h = blockIdx.y;
    uint w = blockIdx.z;
    uint c = threadIdx.x;

    int idx = n*C*H*W + c*H*W + h*W + w;
    output[idx] *= scaler[c];
}

__global__ void kernel_print(float* output)
{
    printf("kernel_print ");
    for (int i = 0; i < 20; ++i)
        printf("%.5f ", output[i]);
    printf("\n");
}
